{
rmVuMeter V 0.94 - component for Delphi.

rmVuMeter is a component for Delphi, written in the Delphi 4
IDE. What about this component and Delphi's versions ?
Delphi 1 : not supported without changes in code
Delphi 2 : certainely works but not tested
Delphi 3 : tested, OK
Delphi 4 : tested, OK

------------------------------------------------------------
Description
------------------------------------------------------------
This component is designed to simulate an audio Vu-Meter.
It can draw single ramp of leds (for mono audio signal
representation) or dual ramp of leds (for stereo audio
signal representation) totaly independant.


------------------------------------------------------------
Actually supported functions
------------------------------------------------------------

AutoSize
--------
When you set AutoSize property to True, Width and Height properties
have no effect. Width and height of rmVuMeter component are determined
by the properties listed bellow:

BorderMargin    - space between rmVuMeter border and leds border
CenterMargin    - space between left ramp and right ramp - only in stereo
ExtremityMargin - space between rmVuMeter extremity and first / last leds
LedLength       - led size: height if rmVuMeter is in vertical orientation
LedSpacing      - space between every leds
LedWidth        - led size: width if rmVuMeter is in vertical orientation
NumLeds         - number of leds

When you set AutoSize property to false (default value), Width and Height
properties determine width and height of rmVuMeter component.
In this case



For this, MeterType property can take four values :
 - vtStereo (left and right values independant)
 - vtMonoL (only left value used for draw)
 - vtMonoR (only right value used for draw)
 - vtMonoLR (left and right values added, and divided by 2)

As real vu-meter, you're not limited to one color (it's right,
you can take a simple gauge and dynamicaly change color -
lime to red for exemple - when desired at execution. I wanted
simpler).
Here, you can decide of two limits (one for prevent and
another for saturation), so three colors can be applied.
Threshold One is fixed by default at 75% and Threshold Two is
fixed by default at 90% of max level.
By defaut, the first color (authorized level) is Lime, second
color (juste before saturation) is Yellow, and thirst color
(saturation) is Red.
Thresholds and color can be changed, but remenber that
Thresholds One must be lower than Thresholds Two. If same value
is specified for the two Thresholds, so rmVuMeter works with
two colors (not three).
Left level and right level are calculated from LeftValue
and RightValue properties.

Orientation property lets you chose orientation (ah ?).

GraphType property, when set to gtBar, specifie that multi-leds
mode must be enabled (as major vu-meters work). When GraphType
property is set to gtSpot, one led at a time in lightened.

You'll find this usefull for representation of one value that
must be "closed" into two values (CAG value in HF receivers for
exemple).

ShaddowType property (representing leds off) can take two values.
When set to stFixed, the color of led off is specified by
ShaddowColor property; when set to stDarkLevelColor, the
color of led off is dependant of color you've assigned to the three
level colors. For exemple, if you set ShaddowType to
stDarkLevelColor, and if you set Level1Color to clLime, the
shaddowColor for this level will be clLime "dark". Of course, when
ShaddowType is set to stDarkLevelColor, ShaddowColor property
is not used.


------------------------------------------------------------
Actually no supported functions (i work on it for next release)
------------------------------------------------------------
Memory peaks
MinValue and MaxValue to extend range


------------------------------------------------------------
Distribution and copyrights
------------------------------------------------------------
As i take initial ideas of multiple sources codes of gauges
(of public domain on internet and CD-ROM) to create this
component, i think it's normal to release it to public domain.

Thanks to all peoples who freely distribute their works (sometimes
hardly obtained).

Special thanks to T Skovmand Eriksen (email
TSEriksen@cyberdude.com), who published "Scope 2.0 the pack" .

Je sais, j'aurais pu joindre ce code source d'une explication
en francais !


------------------------------------------------------------
Any suggestions, comments, or bugs report
------------------------------------------------------------
MALLARD Remy
25 rue de Provence
78310 MAUREPAS
FRANCE

E-Mail: remy.mallard@cetec.tdf.fr


------------------------------------------------------------
Component History
------------------------------------------------------------
Rev 1 : 10/03/1999
Version : 0.94

}


unit rmVumeter;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs,  ExtCtrls;

type
  TVuMeterType = (vtStereo, vtMonoL, vtMonoR, vtMonoLR);
  TOrientation = (voVertical, voHorizontal);
  TShaddowType = (stFixed, stDarkLevelColor);
  TGraphType = (gtBar, gtSpot);

  TrmVuMeter = class(TGraphicControl)
  private
    FVuMeterType: TVuMeterType;
    Forientation: TOrientation;
    FShaddowType: TShaddowType;
    FGraphType: TGraphType;
    FBmpBuffer: TBitmap;
    FLedDisabledColor: TColor;
    FBackGroundColor, FShaddowColor,
    FLevel1Color, FLevel2Color, FLevel3Color: TColor;
    FLevel1Limit, FLevel2Limit: Integer;
    FLeftValue, FRightValue: Integer;
    FBorderMargin, FCenterMargin, FExtremityMargin: integer;
    FLedLength, FLedWidth, FLedSpacing: integer;
    FAutoSize: boolean;
    FNumLeds: integer;

    procedure SetShaddowType(value: TShaddowType);
    procedure SetVuMeterType(value: TVuMeterType);
    procedure SetOrientation(value: TOrientation);
    procedure SetGraphType(value: TGraphType);
    procedure SetLeftValue(value: integer);
    procedure SetRightValue(value: integer);
    procedure SetBackGroundColor(value: TColor);
    procedure SetLedDisabledColor(value: TColor);
    procedure SetShaddowColor(value: TColor);
    procedure SetLevel1Color(value: TColor);
    procedure SetLevel2Color(value: TColor);
    procedure SetLevel3Color(value: TColor);
    procedure SetLevel1Limit(value: integer);
    procedure SetLevel2Limit(value: integer);
    procedure SetBorderMargin(value: integer);
    procedure SetCenterMargin(value: integer);
    procedure SetExtremityMargin(value: integer);
    procedure SetLedLength(value: integer);
    procedure SetLedWidth(value: integer);
    procedure SetLedSpacing(value: integer);
    procedure SetAutoSize(value: boolean);
    procedure SetNumLeds(value: integer);
  protected
    procedure Paint; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure UpdateBmpBuffer;
  public
    constructor Create(anOwner:TComponent); override;
    destructor Destroy; override;
    procedure Free;
  published
    property MeterType: TVuMeterType read FVuMeterType write SetVuMeterType default vtStereo;
    property Orientation: TOrientation read FOrientation write SetOrientation default voVertical;
    property ShaddowType: TShaddowType read FShaddowType write SetShaddowType default stDarkLevelColor;
    property GraphType: TGraphType read FGraphType write SetGraphType default gtBar;
    property LeftValue: integer read FLeftValue write SetLeftValue;
    property RightValue: integer read FRightValue write SetRightValue;
    property LedDisabledColor: TColor read FLedDisabledColor write SetLedDisabledColor;
    property BackGroundColor: TColor read FBackGroundColor write SetBackGroundColor;
    property ShaddowColor: TColor read FShaddowColor write SetShaddowColor;
    property Level1Color: TColor read FLevel1Color write SetLevel1Color;
    property Level2Color: TColor read FLevel2Color write SetLevel2Color;
    property Level3Color: TColor read FLevel3Color write SetLevel3Color;
    property Level1Limit: integer read FLevel1Limit write SetLevel1Limit;
    property Level2Limit: integer read FLevel2Limit write SetLevel2Limit;
    property BorderMargin: integer read FBorderMargin write SetBorderMargin;
    property CenterMargin: integer read FCenterMargin write SetCenterMargin;
    property ExtremityMargin: integer read FExtremityMargin write SetExtremityMargin;
    property LedLength: integer read FLedLength write SetLedLength;
    property LedWidth: integer read FLedWidth write SetLedWidth;
    property LedSpacing: integer read FLedSpacing write SetLedSpacing;
    property AutoSize: boolean read FAutoSize write SetAutoSize;
    property NumLeds: integer read FNumLeds write SetNumLeds;
    { Standard properties }
    property Enabled;
    property Height;
    property Width;
    { Standard events }
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('OptoElec', [TrmVuMeter]);
end;

function GetShaddowColor(cl: TColor): TColor;
type
  colorConvert = record
    case byte of
      0 : (z: TColor);
      1 : (a, b, c, d: byte);
  end;
begin
  colorConvert(result).a := colorConvert(cl).a div 2;
  colorConvert(result).b := colorConvert(cl).b div 2;
  colorConvert(result).c := colorConvert(cl).c div 2;
  colorConvert(result).d := colorConvert(cl).d div 2;
end;

constructor TrmVuMeter.Create(anOwner:TComponent);
begin
  Inherited Create(anOwner);
  Height := 196;
  Width := 20;
  FVuMeterType := vtStereo;
  FOrientation := voVertical;
  FShaddowType := stDarkLevelColor;
  FGraphType := gtBar;
  FLeftValue := 1;
  FRightValue := 1;
  FLevel1Color := clLime;
  FLevel2Color := clYellow;
  FLevel3Color := clRed;
  FLevel1Limit := 75;
  FLevel2Limit := 92;
  FBackGroundColor := clBlack;
  FShaddowColor := clGray;
  FBmpBuffer := TBitmap.Create;
  FBorderMargin := 3;
  FCenterMargin := 4;
  FExtremityMargin := 3;
  FLedDisabledColor := clSilver;
  FLedLength := 10;
  FLedWidth := 5;
  FLedSpacing := 2;
  FAutoSize := false;
  FNumLeds := 16;
end;

destructor TrmVuMeter.Destroy;
begin
  If FBmpBuffer <> NIL then FBmpBuffer.free;
  Inherited Destroy;
end;

procedure TrmVuMeter.Free;
begin
  FBmpBuffer.Free;
  Inherited Free;
end;

procedure TrmVuMeter.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
  end;
end;

procedure TrmVuMeter.UpdateBmpBuffer;

  function ShaddowColor(zone1, zone2, zone3: boolean): TColor;
  begin
    if FShaddowType = stFixed then result := FShaddowColor
    else begin
      if zone1 then result := GetShaddowColor(FLevel3Color);
      if zone2 then result := GetShaddowColor(FLevel2Color);
      if zone3 then result := GetShaddowColor(FLevel1Color);
    end;
  end;

  function BarColor(i, val: integer; zone1, zone2, zone3: boolean): TColor;
  begin
    if enabled then begin
      if ((i >= Val) and zone1) then result := FLevel3Color
      else if ((i >= Val) and zone2) then result := FLevel2Color
      else if ((i >= Val) and zone3) then result := FLevel1Color
      else result := shaddowColor(zone1, zone2, zone3);
    end else begin
      result := FLedDisabledColor;
    end;
  end;

  function SpotColor(i, val: integer; zone1, zone2, zone3: boolean): TColor;
  begin
    if enabled then begin
      if ((i = Val) and zone1) then result := FLevel3Color
      else if ((i = Val) and zone2) then result := FLevel2Color
      else if ((i = Val) and zone3) then result := FLevel1Color
      else result := shaddowColor(zone1, zone2, zone3);
    end else begin
      result := FLedDisabledColor;
    end;
  end;

var
  i, center, pos, LVal, RVal, NumLeds,
  limit1, limit2: integer;
  stereo, vert, spot: boolean;
  zone1, zone2, zone3: boolean;
  TempSize1, TempSize2: integer;
begin
  stereo := (FVuMeterType = vtStereo);
  vert := (FOrientation = voVertical);
  spot := (FGraphType = gtSpot);

  // if AutoSize, rmVuMeter size is determined by NumLeds, LedLength, LedWidth,
  // LedSpacing, BorderMargin, CenterMargin and ExtremityMargin properties,
  // and cannot be set manually (in conception or execution).
  if AutoSize and not(csReading	in ComponentState) then begin
    TempSize1 := (2 * FBorderMargin) + (2 * LedWidth) + FCenterMargin;
    TempSize2 := (2 * FExtremityMargin) + (FNumLeds * FLedLength) + ((FNumLeds - 1) * FLedSpacing);
    if round(TempSize1 / 2) <> (TempSize1 / 2) then tempSize1 := tempSize1 + 1;
    if vert then begin
      height := TempSize2;
      width := tempSize1;
    end else begin
      height := TempSize1;
      width := TempSize2;
    end;
  end;

  With FBmpBuffer.Canvas do begin
    Brush.Color := FBackGroundColor;
    Pen.Color := FBackGroundColor;
    Rectangle(0, 0, width, height);
    if vert then begin
      NumLeds := ((height - (2 * FExtremityMargin)) div (FLedLength + FLedSpacing)) - 1;
      // if FAutoSize then inc(NumLeds);
      inc(NumLeds);
      center := width div 2;
    end else begin
      NumLeds := ((width - (2 * FExtremityMargin)) div (FLedLength + FLedSpacing)) - 1;
      // if FAutoSize then inc(NumLeds);
      inc(NumLeds);
      center := height div 2;
    end;
    if FVuMeterType = vtStereo then begin
      LVal := round((NumLeds / 100) * (100 - LeftValue));
      RVal := round((NumLeds / 100) * (100 - RightValue));
    end else if FVuMeterType = vtMonoL then begin
      LVal := round((NumLeds / 100) * (100 - LeftValue));
    end else if FVuMeterType = vtMonoR then begin
      LVal := round((NumLeds / 100) * (100 - RightValue));
    end else if FVuMeterType = vtMonoLR then begin
      LVal := round((NumLeds / 100) * (100 - ((LeftValue + RightValue) div 2)));
    end;

    limit1 := round((NumLeds / 100) * (100 - FLevel1Limit));
    limit2 := round((NumLeds / 100) * (100 - FLevel2Limit));

    For i := 0 to NumLeds do begin
      if vert then pos := ((FLedLength + FLedSpacing) * i) + FExtremityMargin
      else pos := ((FLedLength + FLedSpacing) * (numLeds - i)) + FExtremityMargin;

      // if (pos + FLedLength + FLedSpacing + FExtremityMargin + 1) > height then break;

      zone1 := (i <= limit2);
      zone2 := ((i > limit2) and (i <= limit1));
      zone3 := (i > limit1);

      // code executed in stereo and mono modes, for left indicator
      if Spot then begin
        pen.Color := spotColor(i, LVal, zone1, zone2, zone3);
        brush.Color := spotColor(i, LVal, zone1, zone2, zone3);
      end else begin
        pen.color := BarColor(i, LVal, zone1, zone2, zone3);
        brush.color := BarColor(i, LVal, zone1, zone2, zone3);
      end;
      // Stereo - Left indicator
      if stereo then begin
        if vert then rectangle(FBorderMargin, pos, center - (FCenterMargin div 2), pos + FLedLength)
        else rectangle(pos, FBorderMargin, pos + FLedLength, center - (FCenterMargin div 2));
      // Mono - Only one indicator
      end else begin
        if vert then rectangle(FBorderMargin, pos, Width - FBorderMargin, pos + FLedLength)
        else rectangle(pos, FBorderMargin, pos + FLedLength, height - FBorderMargin);
        continue;
      end;

      // code executed only in stereo mode, for right indicator
      if Spot then begin
        pen.Color := spotColor(i, RVal, zone1, zone2, zone3);
        brush.Color := spotColor(i, RVal, zone1, zone2, zone3);
      end else begin
        pen.color := BarColor(i, RVal, zone1, zone2, zone3);
        brush.color := BarColor(i, RVal, zone1, zone2, zone3);
      end;
      if vert then rectangle(center + (FCenterMargin div 2), pos, Width - FBorderMargin, pos + FLedLength)
      else rectangle(pos, center + (FCenterMargin div 2), pos + FLedLength, height - FBorderMargin);
    end;
  end;
end;

procedure TrmVuMeter.Paint;
begin
  UpdateBmpBuffer;
  Canvas.Draw(0, 0, FBmpBuffer);
end;

procedure TrmVuMeter.SetBackGroundColor(value: TColor);
begin
  FBackGroundColor := value;
  // UpdateBmpBuffer;
  Paint;
end;

procedure TrmVuMeter.SetLedDisabledColor(value: TColor);
begin
  FLedDisabledColor := value;
  // UpdateBmpBuffer;
  Paint;
end;

procedure TrmVuMeter.SetShaddowColor(value: TColor);
begin
  FShaddowColor := value;
  // UpdateBmpBuffer;
  Paint;
end;

procedure TrmVuMeter.SetLevel1Color(value: TColor);
begin
  FLevel1Color := value;
  // UpdateBmpBuffer;
  Paint;
end;

procedure TrmVuMeter.SetLevel2Color(value: TColor);
begin
  FLevel2Color := value;
  // UpdateBmpBuffer;
  Paint;
end;

procedure TrmVuMeter.SetLevel3Color(value: TColor);
begin
  FLevel3Color := value;
  // UpdateBmpBuffer;
  Paint;
end;

procedure TrmVuMeter.SetLevel1Limit(value: integer);
begin
  If value > 100 then value := 100;
  If value < 0 then value := 0;
  if value > FLevel2Limit then value := FLevel2Limit;
  FLevel1Limit := Value;
  Paint;
end;

procedure TrmVuMeter.SetLevel2Limit(value: integer);
begin
  If value > 100 then value := 100;
  If value < 0 then value := 0;
  if value < FLevel1Limit then value := FLevel1Limit;
  FLevel2Limit := Value;
  Paint;
end;

procedure TrmVuMeter.SetLeftValue(value: integer);
begin
  if value <> FLeftValue then begin
    If value > 100 then value := 100;
    If value < 0 then value := 0;
    FLeftValue := Value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetRightValue(value: integer);
begin
  if value <> FRightValue then begin
    If value > 100 then value := 100;
    If value < 0 then value := 0;
    FRightValue := Value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetVuMeterType(value: TVuMeterType);
begin
  if value <> FVuMeterType then
  begin
    FVuMeterType := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetOrientation(value: TOrientation);
var
  oldWidthValue: integer;
begin
  if value <> FOrientation then
  begin
    FOrientation := value;
    if (csDesigning in ComponentState) and not(csReading in ComponentState) then begin
      oldWidthValue := width;
      width := height;
      height := oldWidthValue;
    end;
    Paint;
  end;
end;

procedure TrmVuMeter.SetShaddowType(value: TShaddowType);
begin
  if value <> FShaddowType then
  begin
    FShaddowType := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetGraphType(value: TGraphType);
begin
  if value <> FGraphType then
  begin
    FGraphType := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetBorderMargin(value: integer);
begin
  if value <> FBorderMargin then
  begin
    FBorderMargin := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetCenterMargin(value: integer);
begin
  if value <> FCenterMargin then
  begin
    FCenterMargin := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetExtremityMargin(value: integer);
begin
  if value <> FExtremityMargin then
  begin
    FExtremityMargin := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetLedLength(value: integer);
begin
  if value <> FLedLength then
  begin
    FLedLength := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetLedWidth(value: integer);
begin
  if value <> FLedWidth then
  begin
    FLedWidth := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetLedSpacing(value: integer);
begin
  if value <> FLedSpacing then
  begin
    FLedSpacing := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetAutoSize(value: boolean);
begin
  if value <> FAutoSize then
  begin
    FAutoSize := value;
    Paint;
  end;
end;

procedure TrmVuMeter.SetNumLeds(value: integer);
begin
  if value <> FNumLeds then
  begin
    if value < 1 then value := 1;
    if value > 1000 then value := 1000;
    FNumLeds := value;
    Paint;
  end;
end;


end.




